# remirepo spec file for php-pecl-mysql-xdevapi
#
# Copyright (c) 2016-2019 Remi Collet
# License: CC-BY-SA
# http://creativecommons.org/licenses/by-sa/4.0/
#
# Please, preserve the changelog entries
#

# we don't want -z defs linker flag
%undefine _strict_symbol_defs_build

%if 0%{?scl:1}
%global sub_prefix %{scl_prefix}
%scl_package       php-pecl-mysql-xdevapi
%endif

%global with_zts    0%{!?_without_zts:%{?__ztsphp:1}}
%global pecl_name   mysql_xdevapi
# Need work to find how to run upstream test suite
%global with_tests  0%{?_with_tests:1}
# After 20-json, 20-mysqlnd
%global ini_name    40-%{pecl_name}.ini

Summary:        MySQL database access functions
Name:           %{?sub_prefix}php-pecl-mysql-xdevapi
Version:        8.0.17
Release:        1%{?dist}%{!?scl:%{!?nophptag:%(%{__php} -r 'echo ".".PHP_MAJOR_VERSION.".".PHP_MINOR_VERSION;')}}

License:        PHP
URL:            http://pecl.php.net/package/%{pecl_name}
Source0:        http://pecl.php.net/get/%{pecl_name}-%{version}%{?prever}.tgz

Patch0:         https://patch-diff.githubusercontent.com/raw/php/pecl-database-mysql_xdevapi/pull/6.patch
Patch1:         https://patch-diff.githubusercontent.com/raw/php/pecl-database-mysql_xdevapi/pull/7.patch

BuildRequires:  %{?dtsprefix}gcc
BuildRequires:  %{?dtsprefix}gcc-c++
BuildRequires:  %{?scl_prefix}php-devel > 7.1
BuildRequires:  %{?scl_prefix}php-mysqlnd
BuildRequires:  %{?scl_prefix}php-hash
BuildRequires:  %{?scl_prefix}php-json
BuildRequires:  %{?scl_prefix}php-pear
BuildRequires:  protobuf-devel
BuildRequires:  protobuf-c-devel
BuildRequires:  boost-devel >= 1.53
BuildRequires:  openssl-devel
%if %{with_tests}
BuildRequires:  community-mysql-server >= 5.7.12
BuildRequires:  %{?scl_prefix}php-mysqli
%endif

Requires:       %{?scl_prefix}php(zend-abi) = %{php_zend_api}
Requires:       %{?scl_prefix}php(api) = %{php_core_api}
Requires:       %{?scl_prefix}php-mysqlnd%{?_isa}
Requires:       %{?scl_prefix}php-json%{?_isa}
Requires:       %{?scl_prefix}php-hash%{?_isa}
%{?_sclreq:Requires: %{?scl_prefix}runtime%{?_sclreq}%{?_isa}}

Provides:       %{?scl_prefix}php-%{pecl_name}               = %{version}
Provides:       %{?scl_prefix}php-%{pecl_name}%{?_isa}       = %{version}
Provides:       %{?scl_prefix}php-pecl(%{pecl_name})         = %{version}
Provides:       %{?scl_prefix}php-pecl(%{pecl_name})%{?_isa} = %{version}
# Notice pecl_name != name
Provides:       %{?scl_prefix}php-pecl-%{pecl_name}          = %{version}-%{release}
Provides:       %{?scl_prefix}php-pecl-%{pecl_name}%{?_isa}  = %{version}-%{release}

# Removed
Obsoletes:      %{name}-devel < 8.0.2

%if "%{?vendor}" == "Remi Collet" && 0%{!?scl:1} && 0%{?rhel}
# Other third party repo stuff
Obsoletes:     php71u-pecl-%{pecl_name} <= %{version}
Obsoletes:     php71w-pecl-%{pecl_name} <= %{version}
%if "%{php_version}" > "7.2"
Obsoletes:     php72u-pecl-%{pecl_name} <= %{version}
Obsoletes:     php72w-pecl-%{pecl_name} <= %{version}
%endif
%if "%{php_version}" > "7.3"
Obsoletes:      php73-pecl-%{pecl_name} <= %{version}
Obsoletes:     php73w-pecl-%{pecl_name} <= %{version}
%endif
%if "%{php_version}" > "7.4"
Obsoletes:      php74-pecl-%{pecl_name} <= %{version}
Obsoletes:     php74w-pecl-%{pecl_name} <= %{version}
%endif
%endif

%if 0%{?fedora} < 20 && 0%{?rhel} < 7
# Filter private shared object
%{?filter_provides_in: %filter_provides_in %{_libdir}/.*\.so$}
%{?filter_setup}
%endif


%description
The X DevAPI is he new common API for MySQL Connectors built on the X Protocol
introduced in MySQL 5.7.12.

The X DevAPI wraps powerful concepts in a simple API.

* A new high-level session concept enables you to write code that
  can transparently scale from single MySQL Server to a multiple server
  environment.
* Read operations are simple and easy to understand.

The X DevAPI introduces a new, modern and easy-to-learn way to work with your
data.

* Documents are stored in Collections and have their dedicated CRUD
  operation set.
* Work with your existing domain objects or generate code based on structure
  definitions for strictly typed languages.
* Focus is put on working with data via CRUD operations. 
  See Section 3.1, “CRUD Operations Overview”.
* Modern practices and syntax styles are used to get away from traditional
  SQL-String-Building. See Chapter 10, Building Expressions.

Documentation: http://php.net/mysql-xdevapi


Package built for PHP %(%{__php} -r 'echo PHP_MAJOR_VERSION.".".PHP_MINOR_VERSION;')%{?scl: as Software Collection (%{scl} by %{?scl_vendor}%{!?scl_vendor:rh})}.


%prep
%setup -qc
mv %{pecl_name}-%{version} NTS

# Don't install/register tests
sed -e 's/role="test"/role="src"/' \
    %{?_licensedir:-e '/LICENSE/s/role="doc"/role="src"/' } \
    -i package.xml

cd NTS
%patch0 -p1 -b .pr6
%patch1 -p1 -b .pr7

# Check version as upstream often forget to update this
extver=$(sed -n '/#define PHP_MYSQL_XDEVAPI_VERSION/{s/.* "//;s/".*$//;p}' php_mysql_xdevapi.h)
if test "x${extver}" != "x%{version}%{?prever}%{?gh_date:-dev}"; then
   : Error: Upstream YAC version is ${extver}, expecting %{version}%{?prever}%{?gh_date:-dev}.
   exit 1
fi
cd ..


%if %{with_zts}
# duplicate for ZTS build
cp -pr NTS ZTS
%endif

# Create configuration file
cat > %{ini_name} << 'EOF'
; Enable %{pecl_name} extension module
extension=%{pecl_name}.so

; Configuration
;xmysqlnd.collect_statistics = 1
;xmysqlnd.collect_memory_statistics = 0
;xmysqlnd.debug = ''
;xmysqlnd.trace_alloc = ''
;xmysqlnd.net_read_timeout = 31536000
;xmysqlnd.mempool_default_size= 16000
EOF


%build
%{?dtsenable}

peclconf() {
%configure \
    --enable-mysql-xdevapi \
    --with-boost \
    --with-protobuf \
    --with-php-config=$1
}

cd NTS
%{_bindir}/phpize
peclconf %{_bindir}/php-config
make %{?_smp_mflags}

%if %{with_zts}
cd ../ZTS
%{_bindir}/zts-phpize
peclconf %{_bindir}/zts-php-config
make %{?_smp_mflags}
%endif


%install
%{?dtsenable}

# Install the NTS stuff
make -C NTS install INSTALL_ROOT=%{buildroot}
install -D -m 644 %{ini_name} %{buildroot}%{php_inidir}/%{ini_name}

# Install XML package description
install -D -m 644 package.xml %{buildroot}%{pecl_xmldir}/%{name}.xml

# Install the ZTS stuff
%if %{with_zts}
make -C ZTS install INSTALL_ROOT=%{buildroot}
install -D -m 644 %{ini_name} %{buildroot}%{php_ztsinidir}/%{ini_name}
%endif

# Documentation
for i in $(grep 'role="doc"' package.xml | sed -e 's/^.*name="//;s/".*$//')
do [ -f NTS/$i ] &&  install -Dpm 644 NTS/$i %{buildroot}%{pecl_docdir}/%{pecl_name}/$i
done


%check
cd NTS

: Minimal load test for NTS extension
%{_bindir}/php --no-php-ini \
    --define extension=mysqlnd.so \
    --define extension=json.so \
    --define extension=%{buildroot}%{php_extdir}/%{pecl_name}.so \
    --modules | grep '^%{pecl_name}$'

%if %{with_zts}
cd ../ZTS

: Minimal load test for ZTS extension
%{__ztsphp} --no-php-ini \
    --define extension=mysqlnd.so \
    --define extension=json.so \
    --define extension=%{buildroot}%{php_ztsextdir}/%{pecl_name}.so \
    --modules | grep '^%{pecl_name}$'

%endif

%if %{with_tests}
cd ../NTS
RET=0

: Running a server
MYSQLX_TEST_HOST=127.0.0.1
MYSQLX_TEST_PORT=3308
MYSQLX_TEST_SOCKET=$PWD/mysql.sock
MYSQLX_PID_FILE=$PWD/mysql.pid

rm -rf data
mkdir  data
%{_libexecdir}/mysqld \
   --datadir=$PWD/data \
   --log-error=$PWD/mysql.log \
   --initialize-insecure

%{_libexecdir}/mysqld \
   --socket=$MYSQLX_TEST_SOCKET \
   --log-error=$PWD/mysql.log \
   --pid-file=$MYSQLX_PID_FILE \
   --port=$MYSQLX_TEST_PORT \
   --datadir=$PWD/data &

n=15
while [ $n -gt 0 ]; do
  RESPONSE=$(%{_bindir}/mysqladmin --no-defaults --socket="$MYSQLX_TEST_SOCKET" --user=root ping 2>&1 || :)
  if [ "$RESPONSE" == "mysqld is alive" ]; then
    break
  fi
  n=$(expr $n - 1)
  sleep 1
done

: Run upstream test suite
export MYSQLX_TEST_HOST
export MYSQLX_TEST_PORT

if [ $n -gt 0 ]; then
TEST_PHP_EXECUTABLE=%{__php} \
TEST_PHP_ARGS="-n -d extension=mysqlnd.so -d extension=json.so -d extension=mysqli.so -d extension=%{buildroot}%{php_extdir}/%{pecl_name}.so" \
NO_INTERACTION=1 \
REPORT_EXIT_STATUS=1 \
%{__php} -n run-tests.php --show-diff || RET=1
fi

: Cleanup
if [ -s $MYSQLX_PID_FILE ]; then
  kill $(cat $MYSQLX_PID_FILE)
fi

exit $RET
%endif


%if 0%{?fedora} < 24 && 0%{?rhel} < 8
# when pear installed alone, after us
%triggerin -- %{?scl_prefix}php-pear
if [ -x %{__pecl} ] ; then
    %{pecl_install} %{pecl_xmldir}/%{name}.xml >/dev/null || :
fi

# posttrans as pear can be installed after us
%posttrans
if [ -x %{__pecl} ] ; then
    %{pecl_install} %{pecl_xmldir}/%{name}.xml >/dev/null || :
fi

%postun
if [ $1 -eq 0 -a -x %{__pecl} ] ; then
    %{pecl_uninstall} %{pecl_name} >/dev/null || :
fi
%endif


%files
%{?_licensedir:%license NTS/LICENSE}
%doc %{pecl_docdir}/%{pecl_name}
%{pecl_xmldir}/%{name}.xml

%config(noreplace) %{php_inidir}/%{ini_name}
%{php_extdir}/%{pecl_name}.so

%if %{with_zts}
%{php_ztsextdir}/%{pecl_name}.so
%config(noreplace) %{php_ztsinidir}/%{ini_name}
%endif


%changelog
* Mon Jul 22 2019 Remi Collet <remi@remirepo.net> - 8.0.17-1
- update to 8.0.17
- adapt patch partially merged upstream

* Tue Jul  9 2019 Remi Collet <remi@remirepo.net> - 8.0.16-2
- add patch for PHP 7.4 from
  https://github.com/php/pecl-database-mysql_xdevapi/pull/6
  https://github.com/php/pecl-database-mysql_xdevapi/pull/7

* Thu Apr 25 2019 Remi Collet <remi@remirepo.net> - 8.0.16-1
- update to 8.0.16

* Fri Feb  1 2019 Remi Collet <remi@remirepo.net> - 8.0.15-1
- update to 8.0.15 (no change)

* Wed Jan 23 2019 Remi Collet <remi@remirepo.net> - 8.0.14-1
- update to 8.0.14

* Mon Oct 22 2018 Remi Collet <remi@remirepo.net> - 8.0.13-1
- update to 8.0.13

* Thu Aug 16 2018 Remi Collet <remi@remirepo.net> - 8.0.12-2
- rebuild for 7.3.0beta2 new ABI

* Fri Aug 10 2018 Remi Collet <remi@remirepo.net> - 8.0.12-1
- update to 8.0.12

* Wed Jul 18 2018 Remi Collet <remi@remirepo.net> - 8.0.11-3
- rebuild for 7.3.0alpha4 new ABI, using patch from
  https://github.com/php/pecl-database-mysql_xdevapi/pull/5

* Fri Jun 29 2018 Remi Collet <remi@remirepo.net> - 8.0.11-2
- add upstream patch for 7.3

* Fri Apr 20 2018 Remi Collet <remi@remirepo.net> - 8.0.11-1
- update to 8.0.11 (stable)

* Thu Feb  1 2018 Remi Collet <remi@remirepo.net> - 8.0.4-1
- Update to 8.0.4 (alpha)

* Fri Nov  3 2017 Remi Collet <remi@remirepo.net> - 8.0.3-1
- Update to 8.0.3 (alpha)

* Tue Jul 18 2017 Remi Collet <remi@remirepo.net> - 8.0.2-2
- rebuild for PHP 7.2.0beta1 new API

* Wed Jul 12 2017 Remi Collet <remi@remirepo.net> - 8.0.2-1
- Update to 8.0.2 (alpha)
- drop devel sub-package
- 32-bit build is broken with GCC < 7

* Tue Mar 14 2017 Remi Collet <remi@remirepo.net> - 1.0.1-1
- Update to 1.0.1 (alpha)

* Thu Dec  8 2016 Remi Collet <remi@fedoraproject.org> - 1.0.0-1
- initial package, version 1.0.0 (alpha)

